/* 
 * Copyright (c) 2001, 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: runtime.h,v 1.9 2002/08/16 12:17:35 malekith Exp $
 */

#ifndef GONT_RUNTIME_H
#define GONT_RUNTIME_H 1

#include <stddef.h>

/*** Basic types. ***/

/* Integer of the same size, as pointer. */
#define gont_ptroff unsigned long

/*** Memory management. ***/

/* All gont_xmalloc* routines prints error message and aborts if there
 * is not enough memory to satisfy request. */

/* Initialize allocator. */
void gont_init_memory_allocator();

/* Allocate given amount of memory. */
void *gont_xmalloc(gont_ptroff s);

/* Allocate given amount of memory and clear it. */
void *gont_xmalloc0(gont_ptroff s);

/* Allocate given amount of memory, one cannot put pointers into this memory 
 * then (for example this is used for strings). */
void *gont_xmalloc_atomic(gont_ptroff s);

/* As above, and clear it. */
void *gont_xmalloc0_atomic(gont_ptroff s);

#define gont_free(x) do {} while (0)


/*** Initialization/finalization. ***/
extern int gont_argc;
extern char **gont_argv;
extern int gont_exit_code;

void gont_init_runtime(void);
void gont_set_argv(int argc, char **argv);
void gont_register_exit_routine(void (*fnc)(void*), void *data);
void gont_run_exit_routines(void);
void gont_exit(void);

/* Print error message and abort. */
void gont_oops(const char *msg, ...) __attribute__ ((__noreturn__));

/*** Exceptions. ***/
struct gont_try_block;		/* Hidden. */

/* Value returned from this function can be used as __builtin_setjmp 
 * buffer. */
struct gont_try_block *gont_gont_try_block(void);

/* Pop value from try-block chain. */
void *gont_pop_raised_value(void);

/* Just pop one block, throw away the value. */
void gont_pop_try_block(void);

/* Raise exception. */
void gont_raise(void *v);

void gont_fail_with(const char *s);
void gont_invalid_argument(const char *s);
void gont_sys_error(gont_ptroff err);


/*** Complex Gont datatypes. ***/

/* Lambda expressions. */
struct gont_lambda_exp {
	void *(*fnc)(void *, ...);	/* Pointer to function. */
	void *closure;			/* Pointer to its closure. */
};

/* Make new lambda expression. */
struct gont_lambda_exp *gont_make_lambda(void *fnc, void *closure);


/* Strings. */
struct gont_string {
	gont_ptroff len;
	char data[0];
};

struct gont_string *gont_make_string(gont_ptroff size, const char *content);

/* Make string out of NUL terminated C string. */
struct gont_string *gont_cstring(const char *s);

gont_ptroff gont_streq(struct gont_string *s1, struct gont_string *s2);
gont_ptroff gont_strcmp(struct gont_string *s1, struct gont_string *s2);
struct gont_string *gont_strjoin(struct gont_string *s1, 
			         struct gont_string *s2);
/* Result is always in [0, 0x7fffffff]. */
gont_ptroff gont_hash_string(struct gont_string *s);

/* Identity function. */
gont_ptroff gont_id(gont_ptroff x);

/* Floats. */
double *gont_make_float(double v);


/* Unions. */
struct gont_union {
	gont_ptroff field_no;
	void *value;
};

struct gont_union *gont_make_union(gont_ptroff field_no, void *value);
void **gont_make_tuple(gont_ptroff n, ...);

#endif /* GONT_RUNTIME_H */
